#! /usr/bin/env perl
#
# Given a device class and name, such as "net/eth0", find
# the corresponding ibm location code.  Do this by looking 
# up the device spec in the /sys tree followed by a lookup
# in the device-tree.

sub get_loc_code
{
	my $device = shift;
	my $syspath = "/sys/class/" . $device . "/device/devspec";

	open (SPEC, $syspath) or die "Can't open $syspath: $!";
	
	my $specpath = <SPEC>;
	close SPEC;

	$specpath = "/proc/device-tree/" . $specpath . "/ibm,loc-code";

	open (LOC, $specpath) or die "Can't open $specpath: $!";

	$loc_code = <LOC>;
	close LOC;

	return $loc_code;
}

if ($#ARGV != 0) {
	print "Usage: $0 deviceclass/devicename\n";
	print "\tFor example:  $0 net/eth0\n";
	exit 1;
}

$loc_code = get_loc_code ($ARGV[0]);

print "$loc_code\n";

