#! /usr/bin/env perl
#
# FILE: pci-inject.pl
# 
# Simple test script to inject EEH errors into the system,
# and then attempt to recover the device by performing 
# hot-plug add and remove.
#
# This is written as a perl script and not a shell script 
# mostly because its easier to write perl than shell scripts.
#
# The $bringup_cmd is used to bring up the device after a hot-plug.
# It should return a zero value if the bringup was sucessful.
#
# The $check_up_cmd is used to see if the device is working.  
# It should be set to something that returns 0 if the device
# is operational, else it should return a non-zero error.
#
# The $check_down_cmd is used to see that the device really is off-line.
# It should be set to something that returns 0 if the device
# is operational, else it should return a non-zero error.
#
# Linas Vepstas <linas@austin.ibm.com> May 2004


$device = "net/eth0";
# $location = "U0.1-P2-I1";   # set below via lookup
# $bringup_cmd = "ifup eth0";
$bringup_cmd = "echo howdy doody";

$check_up_cmd = "ping -c 5 -q 9.3.117.3";
$check_down_cmd = "ping -c 5 -q 9.3.117.3";

$niterations = 5000;

@err_type_mask = (2,3,8,9,12,13,14,15,16,17,18,19);

# ------------------------------------------------------------
use POSIX qw(ctime);
# ------------------------------------------------------------
# This subroutine returns the location code that corresponds
# to the indicated device
sub get_loc_code
{
	my $device = shift;
	my $syspath = "/sys/class/" . $device . "/device/devspec";

	open (SPEC, $syspath) or die "Can't open $syspath: $!";
	
	my $specpath = <SPEC>;
	close SPEC;

	$specpath = "/proc/device-tree/" . $specpath . "/ibm,loc-code";

	open (LOC, $specpath) or die "Can't open $specpath: $!";

	$loc_code = <LOC>;
	close LOC;

	return $loc_code;
}

# ------------------------------------------------------------
$location = get_loc_code ($device);

# ------------------------------------------------------------

for ($i=0; $i<20; $i++)
{
	$do_inject[$i] = 0;
}

$k = pop @err_type_mask;
while ($k) {
	$do_inject[$k] = 1;
	print "Will inject error type $k\n";
	$k = pop @err_type_mask;
}

# ------------------------------------------------------------
# Main device test loop
#
$max_inject_tries = 100;
for ($n=0; $n<$niterations; $n++)
{
for ($i=0; $i<20; $i++)
{
	# skip the errors we don't want to inject
	if ($do_inject[$i] == 0)  { next;  }
	print "================================ Begin repetition $n; going to inject error code $i\n";

	# check to see if a hotplug event magically showed up very late in 
	# the previous iteration.
	$file_present = 1;
	open (BOGUS, "/tmp/hot-down") or $file_present = 0;
	if ($file_present == 1) { 
		$late_event ++;
	}

	# keep repeating the error until the /etc/hotplug/net.agent
	# script generates output to the /tmp/hot-down file
	system ("/bin/rm -f /tmp/hot-down; sleep 1");
	$finj = 0;
	$k = 0;
	while ($k<$max_inject_tries) {
		$rc = system ("/root/bin/errinjct ioa-bus-error -f $i -s $device 2>&1 >>/dev/null");
		$file_present = 1;
		open (BOGUS, "/tmp/hot-down") or $file_present = 0;
		if ($file_present == 1) { last; }
		$k ++;

		# If the errinject command failed, but the hotplug script
		# hasn't yet run, then sleep, and give it a chance.
		if ($rc) {
			system ("sleep 3");
		}
	}
	system ("cat /proc/ppc64/eeh");

	# give it a last few seconds for the file to materialize, and then 
	# complain ... 
	# Make a note of every time we fail to get the hotplug event.
	system ("sleep 3");
	open (BOGUS, "/tmp/hot-down") or $file_present = 0;
	if ($file_present == 0) { 
		print "Warning: failed to receive a hotplug event\n";
		$failed_inject[$i] ++;
		$finj = 1;
	}

	# system should be down now ... 
	system ("sleep 3");
	$rc = system ("$check_down_cmd");
	if ($rc == 0) 
	{
		print "Warning: system still functional after errinjct\n";
		$failed_down[$i] ++;

		# it didn't go down, *and* its still pingable; 
		# go inject the next error
		next;
	}

	# If we failed to inject an error (no EEH event detected)
	# but the network connection is down, then try to see if
	# a simple interface reset will clear the error.  But first,
	# see if simply waiting is enough to recover from this.
	if ($finj) {

		$non_eeh_err[$i] ++;

		# sometimes, it seems to take the scripts a while to get going.
		for ($k=0; $k<10; $k++) {
			$rc = system ("$check_up_cmd");
			if ($rc == 0) { last; }
			system ("echo waiting 5 for auto-up...");
			system ("sleep 5");
			$slow_auto_up[$i] += 5;
		}

		if ($k < 10) {
			print "Auto-Recovered from non-eeh error\n";
			$auto_recovered_non[$i] ++;

			# go inject the next error
			next;
		}

		# ten tries, 5 seconds each, still not up.
		$slow_auto_up[$i] -= 50;

		# if we are here' then waiting for auto-recovery didn't 
		# do the trick. Manuall cycle the ethernet interface.
		system ("ifdown eth0");
		system ("sleep 1");
		system ("ifup eth0");
		system ("sleep 1");
		$rc = system ("$check_down_cmd");
		if ($rc == 0) 
		{
			print "Manually Recovered from non-eeh error\n";
			$recovered_non[$i] ++;

			# go inject the next error
			next;
		}
	}

	# Bring the slot down and back up again.
	system ("/root/bin/drslot_chrp_pci -n -r -s $location");

	$rc = system ("/root/bin/drslot_chrp_pci -n -a -s $location");
	if ($rc != 0) 
	{
		# sometimes, the first hot-add fails, mysteriously.
		# we try again, for good luck.
		print "first attempt to hot-add failed, trying again\n";
		$rc = system ("/root/bin/drslot_chrp_pci -n -a -s $location");
		if ($rc != 0) 
		{
			print "second attempt to up failed\n";
			$two_up[$i] ++;
		}
		$failed_up[$i] ++;
	}

	# Additional command to bring up the corresponding subsystem
	$rc = system ("$bringup_cmd");
	if ($rc != 0) 
	{
		print "device can't be brought up, \"$bringup_cmd\" failed, rc = $rc\n";
		exit (1);
	}

	# sometimes, it seems to take the scripts a while to get going.
	for ($k=0; $k<10; $k++) {
		$rc = system ("$check_up_cmd");
		if ($rc == 0) { last; }
		system ("echo sleeping 5...");
		system ("sleep 5");
		$slow_up[$i] += 5;
	}

	# testing 1,2,3 do you read me; make sure the subsystem works.
	if ($rc != 0) 
	{
		$broken_up[$i] ++;
		$slow_up[$i] -= 50;

		print "device not usable, \"$check_up_cmd\" failed, rc = $rc\n";

		# The system seems to suffer harder with certain errors. 
		# so try to recover a little bit harder.
		system ("/root/bin/drslot_chrp_pci -n -r -s $location");
		system ("/root/bin/drslot_chrp_pci -n -a -s $location");
		system ("sleep 25");
		system ("$bringup_cmd");
		$rc = system ("$check_up_cmd");
		if ($rc != 0) 
		{
			print "device still not usable, \"$check_cmd\" failed, rc = $rc\n";
			exit (1);
		}
	} else {
		$hot_up[$i] ++;
	}
}

print "e == error number that was injected\n";
print "fld-inj == hotplug down scripts failed to run (probably due to no eeh error)\n";
print "fld-dwn == interface still works after mutlple error injection attempts\n";
print "nee == non-eeh-error; interface is down, but not due to eeh error\n";
print "auto-rcov == number of times the system auto-recovered itself\n";
print "auto-tm == avg. number of seconds to auto-recovery\n";
print "rcov == number of times the system recovered after manual interface cycle\n";
print "fld-up == number of times that a slot hot-add failed\n";
print "two-up == number of times that a second attempted slot hot-add failed\n";
print "hot-up == number of times that hotplug add/remove recovered slot\n";
print "slw-up == avg num seconds till device came back\n";
print "brkn-up == number of times that hotplug toggle failed to recover device\n";

print "number of late events=$late_event\n";

for ($j=0; $j<20; $j++) {
	# skip the errors we don't want to inject
	if ($do_inject[$j] == 0)  { next;  }

	if ($auto_recovered_non[$j]) { 
		$slo_aup = $slow_auto_up[$j] / $auto_recovered_non[$j];
	} else {
		$slo_aup = "";
	}
	if ($hot_up[$j]) {
		$slo_mup = $slow_up[$j] / $hot_up[$j];
	} else {
	$slo_mup = "";
	}

	print "e=$j fld-inj=$failed_inject[$j] fld-dn=$failed_down[$j] nee=$non_eeh_err[$j] auto-rcov= $auto_recovered_non[$j] auto-tm=$slo_aup rcov=$recovered_non[$j] fld-up=$failed_up[$j] two-fup=$two_up[$i] hot-up=$hot_up[$j] slw-up=$slo_mup brkn-up=$broken_up[$j]\n";
	}
$s = ctime (time);
print "$s";
}

# ----------------------------- END OF FILE --------------------------
